
local RW = LibStub("AceAddon-3.0"):GetAddon("Raid Watch")
local plugin = RW:Plugin("Options")

----------------------------
-- Locals

local ACD = LibStub("AceConfigDialog-3.0")
local LoD = RW:GetPlugin("LoD")
local AceGUI = LibStub("AceGUI-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("RW2-Options")
local LB = LibStub("AceLocale-3.0"):GetLocale("RW2-Plugin-Base")
local LSM = LibStub("LibSharedMedia-3.0")
local UIF = LibStub("LibGUIFactory-1.0"):GetFactory("RW")
local LBB
UIF:SetValuesChangedCallback(function()
	RW.Callbacks:Fire("RefreshOptions")	
end)

local Utils = RW.Utils

local OptionsFrame
local DetailsFrame
local OptionsTree

local AceOptionData = {}
local GUIOptionsData = {}
local GUIBossMods = {}
local GUIBossSubGroups = {}

local defaults = {
	profile = {
		VersionSort = "name",
		TestOpt = true,
		Color = {1, 0, 0, 1},
		WindowButtonSize = 20,
		WindowTitleColor = {0/255, 79/255, 173/255, 1},
		WindowFont = "accid",
		WindowFontSize = 18,		
		WindowFontFlags = "NONE",		
		WindowFontColor = {1, 1, 1, 1},
	}
}

----------------------------
-- Init

function plugin:OnRegister()
	self.db = RW.db:RegisterNamespace("Options", defaults)
	
	self.BossOptions = {BossMods = {}}
	self.PluginOptions = {Plugins = {}}
	
	self.db.RegisterCallback(self, "OnProfileChanged", "Reset")
	self.db.RegisterCallback(self, "OnProfileCopied", "Reset")
	self.db.RegisterCallback(self, "OnProfileReset", "Reset")
	
	self:RegisterCallback("RefreshOptions")
	
	self.options = {
		type = "group",
		name = "Raid Watch 2",
		handler = self,
		args = {			
		}
	}	
end

function plugin:PluginEnable()
	LBB = RW.LBB
	self.NextConfigMode = true
	LibStub("AceConfig-3.0"):RegisterOptionsTable("RW", self.options)
	
	self:RegisterCallback("BossModLoaded")
end

function plugin:PluginDisable()
end

function plugin:Reset()
	Utils:ApplyWindowSettings()
end

function plugin:ToggleConfigMode()
	RW.Callbacks:Fire("ConfigMode", self.NextConfigMode)	 
	self.NextConfigMode = not self.NextConfigMode
end

----------------------
-- Message handlers --
----------------------

function plugin:BossModLoaded(message, modName)
	RW.Callbacks:Fire("RefreshOptions")
end

----------------------

do

	local treeStatusTable = {}

	-----------------------------
	-- Main window
	
	function plugin:Toggle()
		if OptionsFrame then
			OptionsFrame:Release()
			OptionsFrame = nil
		else	
			OptionsFrame = UIF:Frame("Raid Watch 2")
			OptionsFrame:SetLayout("Flow")
			OptionsFrame:SetWidth(810)
			OptionsFrame:SetHeight(620)
			OptionsFrame:SetCallback("OnClose", function(widget)
				widget:ReleaseChildren()
				widget:Release()
				if DetailsFrame then
					DetailsFrame:Release()
					DetailsFrame = nil
				end
				OptionsFrame = nil
			end)
			
			OptionsFrame.statustext:SetFont(LSM:Fetch("font", "accid"), 17, "OUTLINE")
			OptionsFrame.statustext:SetText("  v"..RWVERSION.." (r"..RWREVISION.. ") "..RWRELEASETPYE)
			
			OptionsTree = UIF:TreeGroup()
			self.OptionsTree = OptionsTree
			OptionsFrame:AddChild(OptionsTree)
			OptionsTree:SetLayout("Fill")
			OptionsTree:SetFullHeight(true)
			
			OptionsTree:AddGroup(self:OptionsMain())
			for i, f in pairs(self:OptionsPlugins()) do
				OptionsTree:AddGroup(f())
			end
			for i, f in pairs(self:OptionsBoss()) do
				OptionsTree:AddGroup(f())
			end
			
			OptionsTree:SetCallback("OnGroupSelected", function(self, event, group) plugin.treeLastSelected = group end)		
			OptionsTree:SetSelected(plugin.treeLastSelected	 or "Main")	
			OptionsTree:SetStatusTable(treeStatusTable)			
			
			OptionsFrame:Show()
			OptionsFrame:DoLayout()
			AceGUI:ClearFocus()
		end
	end
	
	function plugin:RefreshOptions(event)
		if OptionsFrame then	
			plugin:Toggle()
			plugin:Toggle()
		end
	end
end