local RW = LibStub("AceAddon-3.0"):GetAddon("Raid Watch")
local plugin = RW:GetPlugin("Options")

local AceGUI 	= LibStub("AceGUI-3.0")
local UIF		= LibStub("LibGUIFactory-1.0"):GetFactory("RW")
local L 		= LibStub("AceLocale-3.0"):GetLocale("RW2-Options")
local LB = LibStub("AceLocale-3.0"):GetLocale("RW2-Plugin-Base")
local Utils 	= RW.Utils

function plugin:OptionsPlugins()
	local LBB = RW.LBB
	local t = {
		function()
			return {{text = LB.PLUGINS, value = "Plugins", order = 10}}, function()
				local self = self or plugin
				rdb = RW.db.profile
				
				local group, button, label, check
				
				local function WindowCallback()
					Utils:ApplyWindowSettings()
				end
				
				group = AceGUI:Create("SimpleGroup")
				group:SetLayout("Flow")
				group:SetFullWidth(true)
				
				button = UIF:Button(L.TOGGLE_CONFIG_MODE, function()
					RW.Callbacks:Fire("ConfigMode", self.NextConfigMode) 
					plugin.NextConfigMode = not plugin.NextConfigMode
				end)	
				group:AddChild(button)
				
				group:AddChild(UIF:CheckBox(L.GLOBAL_BOSS_HEALTH_FRAMES, rdb, "GlobalBossHealthFrames", nil, L.GLOBAL_BOSS_HEALTH_FRAME_DESC))
				group:AddChild(UIF:CheckBox(L.USE_CLEAN_MESSAGES, rdb, "UseCleanMessages", nil, L.USE_CLEAN_MESSAGES_DESC))
				
				local ig = UIF:InlineGroup1(L.WINDOW_SETTINGS)
				group:AddChild(ig)
				ig:AddChild(UIF:Text1(L.WINDOW_SETTINGS_DESC, nil, nil, nil, nil, nil, 0.6))
				ig:AddChild(UIF:NewLine(10))
				
				local ig2 = UIF:InlineGroup2(L.TITLE)
				ig:AddChild(ig2)	
				
				ig2:AddChild(UIF:CheckBox(L.GRADIENT, rdb, "WindowUseGradient", WindowCallback, L.GRADIENT_DESC, 0.33))
				ig2:AddChild(UIF:Spacer(20))
				ig2:AddChild(UIF:ColorSelect(LB.COLOR, rdb, "WindowTitleColor", WindowCallback, true, 0.20))		
				ig2:AddChild(UIF:Slider(LB.HEIGHT, rdb, "WindowTitleHeight", 5, 200, 1, WindowCallback, 0.33))	
				ig2:AddChild(UIF:LSMDropdown("font", LB.FONT, rdb, "WindowFont", WindowCallback, 0.4))	ig2:AddChild(UIF:Spacer(20))	
				ig2:AddChild(UIF:Dropdown(LB.FONT_OUTLINE, rdb, "WindowFontFlags", WindowCallback, Utils.Constants.FontFlagList, 0.4))
				ig2:AddChild(UIF:Slider(LB.FONT_SIZE, rdb, "WindowFontSize", 1, 50, 1, WindowCallback, 0.4))	
				ig2:AddChild(UIF:Spacer(20))
				ig2:AddChild(UIF:ColorSelect(LB.FONT_COLOR, rdb, "WindowFontColor", WindowCallback, nil, 0.4))
				
				
				local ig2 = UIF:InlineGroup2(L.BUTTONS)
				ig:AddChild(ig2)	
				
				ig2:AddChild(UIF:ColorSelect("Button color", rdb, "WindowButtonColor", WindowCallback, true))
				ig2:AddChild(UIF:Slider("Button size", rdb, "WindowButtonSize", 5, 100, 1, WindowCallback))
				
				return group
			end
		end,
		
		function()
			return {{text = "Profiles", value = "Profiles", order = 20}}, function() return UIF:AceProfileOption(RW.db) end
		end,
		
		function()
			return {{text = LB.PLUGINS, value = "Plugins", order = 10}, { text = LB.ALERT_PLUGINS, value = "AlertPlugins", order = 0}}, function()
				local self = self or plugin
	
				local group, button, label
				
				group = AceGUI:Create("SimpleGroup")
				group:SetLayout("Flow")
				group:SetFullWidth(true)
				
				label = UIF:NormalText(LB.ALERT_PLUGINS_DESC)
				label:SetImage([[Interface\Icons\INV_Sigil_UlduarAll]])
				group:AddChild(label)
				
				return group
			end,
			[[Interface\Icons\INV_Sigil_UlduarAll]]
		end,
		
		function()
			return {{text = LB.PLUGINS, value = "Plugins", order = 10}, { text = LB.BOSS_PLUGINS, value = "BossPlugins", order = 0}}, function()
				local self = self or plugin
	
				local group, button, label
				
				group = AceGUI:Create("SimpleGroup")
				group:SetLayout("Flow")
				group:SetFullWidth(true)
				
				label = UIF:NormalText(LB.BOSS_PLUGINS_DESC)
				label:SetImage([[Interface\Icons\Ability_Druid_ForceofNature]])
				group:AddChild(label)
				
				return group
			end,
			[[Interface\Icons\Ability_Druid_ForceofNature]]
		end,
	}
	
	local all = {}	
	for _, module in RW:IteratePlugins() do
		if module.GetOptions then				
			local p, func, icon = module:GetOptions()
			if p.multiOptions then
				for i, subPath in ipairs(p) do	
					tinsert(all, {
						opt = subPath.path,
						func = subPath.func,
						icon = subPath.icon,
					})
				end
			else
				tinsert(all, {
					opt = p,
					func = func,
					icon = icon,
				})
			end			
		end
	end
	
	for i, d in pairs(all) do
		tinsert(t, function()
			return d.opt, d.func, d.icon
		end)
	end
		
	
	
	return t
end