
----------------------------------
-- Create plugin
local plugin = RW:Plugin("VersionDisplay")
if not plugin then return end

----------------------------------
-- Locals 
local AceGUI = LibStub("AceGUI-3.0")
local LSM = LibStub("LibSharedMedia-3.0")
local LB = LibStub("AceLocale-3.0"):GetLocale("RW2-Plugin-Base")
local L = LibStub("AceLocale-3.0"):GetLocale("RW2-Plugin-VersionCheck")
local UIF
local db
local ConfigWindow
local GuildVersions = {}
local PartyVersions = {}
local Party
local UpdateSchedule, SortFunctions

local PluginIcon = [[Interface\Icons\Achievement_BG_3flagcap_nodeaths]]

local defaults = {
	profile = {
		PluginEnabled = true,
		
		ColorSameVersion = {0, 1, 0},
		ColorOlderVersion = {1, 0, 0},
		ColorNewVersion = {1, 0.5, 0},
		AlternatingRowColor = {0, 0, 0, 0.5},
		SortMethod = "REVISION",
	}
}


----------------------------------
-- Init

function plugin:OnRegister()
	self.db = RW.db:RegisterNamespace("VersionDisplay", defaults)
	db = self.db.profile
	self.db.RegisterCallback(self, "OnProfileChanged", "Reset")
	self.db.RegisterCallback(self, "OnProfileCopied", "Reset")
	self.db.RegisterCallback(self, "OnProfileReset", "Reset")	
end

function plugin:PluginEnable()
	if not db.PluginEnabled then
		self:Disable()
		return
	end
	
	Party = RW.Party
	
	if IsInGuild() then
		self:ScheduleTimer("SendVersion", 4)
		RW.Com:RegisterListener(self, "IncGuildVersionInfo")	
		self.GuildVersions = {}
	end
	
	RW.Com:RegisterListener(self, "IncVersionInfo")
	RW.Com:RegisterListener(self, "ReqVersionInfo")	
	
	self:RegisterCallback("VersionsShow")
	
	UIF = LibStub("LibGUIFactory-1.0"):GetFactory("RW")
end

function plugin:PluginDisable()
	RW.Com:UnregisterAllListeners(self)	
end

function plugin:Reset()
	db = self.db.profile
end

-- COM Handlers ---------------------
-------------------------------------
function plugin:SendVersion()
	RW.Com:Send("IncGuildVersionInfo", "GUILD", nil, RWVERSION, RWREVISION, RWRELEASETPYE)
end

function plugin:IncGuildVersionInfo(sender, dist, version, revision, releaseType)
	GuildVersions[sender] = self.GuildVersions[sender] or {}
	GuildVersions[sender].name = sender
	GuildVersions[sender].version = tonumber(version)
	GuildVersions[sender].revision = tonumber(revision)
	GuildVersions[sender].releaseType = releaseType
	
	if sender ~= UnitName("player") and dist == "GUILD" then
		RW.Com:Send("IncGuildVersionInfo", "WHISPER", sender, RWVERSION, RWREVISION, RWRELEASETPYE)
	end
	
	if UpdateSchedule then self:CancelTimer(UpdateSchedule, true) end
	UpdateSchedule = self:ScheduleTimer("Update", 3)
end

function plugin:IncVersionInfo(sender, dist, version, revision, releaseType)
	if Party.group[sender] then
		Party.group[sender].version = tonumber(version)
		Party.group[sender].revision = tonumber(revision)
		Party.group[sender].releaseType = releaseType		
	end
	
	if UpdateSchedule then self:CancelTimer(UpdateSchedule, true) end
	UpdateSchedule = self:ScheduleTimer("Update", 3)
end

function plugin:ReqVersionInfo(sender, dist)
	RW.Com:Send("IncVersionInfo", "WHISPER", sender, RWVERSION, RWREVISION, RWRELEASETPYE)
end

function plugin:Update()
	self = plugin
	UpdateSchedule = nil
	if ConfigWindow and ConfigWindow.frame:IsShown() then
		plugin:ShowWindow(true)
	end
end

-- Message Handlers ----------
------------------------------
function plugin:VersionsShow(event)
	self:ShowWindow()
end

-- Other ---------------------
------------------------------

local ColorTable = {}
function plugin:GetClassColor(InName)
	wipe(ColorTable)
	if Party.group[InName] then
		local c = RAID_CLASS_COLORS[Party.group[InName].class]
		ColorTable[1] = c.r
		ColorTable[2] = c.g
		ColorTable[3] = c.b
	else
		for i=1, GetNumGuildMembers(true) do
			name, rank, rankIndex, level, class, zone, note, officernote, online, status, classFileName = GetGuildRosterInfo(i)
			if name == InName then
				local c = RAID_CLASS_COLORS[classFileName]
				ColorTable[1] = c.r
				ColorTable[2] = c.g
				ColorTable[3] = c.b
			end
		end
	end
	
	return ColorTable[1] and ColorTable or nil
end

SortFunctions = {}
SortFunctions["REVISION"] = function(a, b)
	if a.revision == b.revision then
		return a.name < b.name
	end
	return a.revision > b.revision
end

SortFunctions["NAME"] = function(a, b)
	return a.name < b.name
end

SortFunctions["VERSION"] = function(a, b)
	if a.version == b.version then
		return SortFunctions["REVISION"](a, b)
	end
	return a.version > b.version
end

-- GUI -----------------------
------------------------------
do
	function plugin:ShowWindow(toggle)
		if not ConfigWindow then
			ConfigWindow = UIF:Frame("Versions")	
			ConfigWindow:SetHeight(800)
			ConfigWindow:SetCallback("OnClose", function()
				ConfigWindow:ReleaseChildren()
				ConfigWindow.frame:Hide()
			end)
			ConfigWindow:SetLayout("Fill")	
			ConfigWindow:Show()
			ConfigWindow:AddChildren(plugin:GetGUI())
			return
		end
		
		if toggle then
			if ConfigWindow.frame:IsShown() then
				ConfigWindow:Hide()
				ConfigWindow:Show()
				ConfigWindow:AddChildren(plugin:GetGUI())
			else
				ConfigWindow:Show()
			end		
		else
			if ConfigWindow.frame:IsShown() then
				ConfigWindow:Hide()
			else		
				ConfigWindow:Show()
				ConfigWindow:AddChildren(plugin:GetGUI())		
			end
		end
	end

	function plugin:GetOptions()
		return {{text = "Versions", value = "Versions"}}, self.GetGUI
	end
	
	local TabSelected
	local LastSelectedTab
	local SortMethods = {
		["REVISION"] = "Revision",
		["NAME"] = "Name",
		["VERSION"] = "Version",
	}

	function plugin:GetGUI(used)
		local self = self or plugin
		
		local scroll = AceGUI:Create("SimpleGroup")
		scroll:SetLayout("Flow")
		scroll:SetFullWidth(true)
		
		label = UIF:MainTitleLabel(L.NAME)
		scroll:AddChild(label)
		
		local group = UIF:InlineGroup("")
		scroll:AddChild(group)	
		
		label = UIF:NormalText(L.DESC)
		label:SetImage(PluginIcon)
		group:AddChild(label)
		
		local tgroup = UIF:TabGroup1()
		scroll:AddChild(tgroup)
		tgroup:SetTabs(
		{
			{text = "Guild", value = "Guild"},
			{text = "Party", value = "Party"},
			{text = "Settings", value = "Settings"},
		})
		tgroup:SetCallback("OnGroupSelected", TabSelected)
		tgroup:SelectTab(LastSelectedTab or "Guild")		
		
		return scroll
	end
	
	local list = {}
	local noVersionList = {}
	function BuildSortedList(source)
		wipe(list)
		noVersionList = {}
		for name, data in pairs(source) do
			if not data.revision or not data.version or not data.releaseType then
				data.releaseType = "None"
				tinsert(noVersionList, data)				
			else
				tinsert(list, data)
			end
		end
		
		table.sort(list, SortFunctions[db.SortMethod])
		
		for i, data in pairs(noVersionList) do
			tinsert(list, data)
		end
		
		return list
	end
	
	TabSelected = function(self, event, tab)
		self:ReleaseChildren()
		LastSelectedTab = tab
		
		local SourceList
		
		if tab == "Guild" then		
			SourceList = GuildVersions
		
		elseif tab == "Party" then
			SourceList = Party.group
			
		elseif tab == "Settings" then
			self:AddChild(UIF:ColorSelect(L.OLD, db, "ColorOlderVersion"))
			self:AddChild(UIF:ColorSelect(L.SAME, db, "ColorSameVersion"))
			self:AddChild(UIF:ColorSelect(L.NEW, db, "ColorNewVersion"))
			self:AddChild(UIF:ColorSelect(L.ROW_ALTERNATE_COLOR, db, "AlternatingRowColor", false, true))			
		end
		
		if tab == "Guild" or tab == "Party" then
		
			local group = UIF:InlineGroup()
			self:AddChild(group)
			local color = UIF:ColorSelect(L.OLD, db, "ColorOlderVersion", nil, nil, 0.2)
			color:SetDisabled(true)
			group:AddChild(color)
			color = UIF:ColorSelect(L.SAME, db, "ColorSameVersion", nil, nil, 0.2)
			color:SetDisabled(true)
			group:AddChild(color)
			color = UIF:ColorSelect(L.NEW, db, "ColorNewVersion", nil, nil, 0.2)
			color:SetDisabled(true)
			group:AddChild(color)
			group:AddChild(UIF:Dropdown(L.SORT_METHOD, db, "SortMethod", plugin.Update, SortMethods, 0.4))
			
			self:AddChild(UIF:Text1("Name", nil, nil, nil, nil, nil, 0.3))
			self:AddChild(UIF:Text1("Revision", nil, nil, nil, nil, nil, 0.2))
			self:AddChild(UIF:Text1("Version", nil, nil, nil, nil, nil, 0.2))
			self:AddChild(UIF:Text1("Release Type", nil, nil, nil, nil, nil, 0.3))
			
			local list = BuildSortedList(SourceList)
			
			local count = 0
			for i, data in pairs(list) do
				count = count + 1
				
				local VersionColor
				if not data.revision then
				elseif data.revision > tonumber(RWREVISION) then VersionColor = db.ColorNewVersion 
				elseif data.revision == tonumber(RWREVISION) then VersionColor = db.ColorSameVersion 
				elseif data.revision < tonumber(RWREVISION) then VersionColor = db.ColorOlderVersion end
				
				local name 		= UIF:Text1(data.name, plugin:GetClassColor(data.name), nil, nil, nil, nil, 0.3)
				local revision = UIF:Text1(data.revision or "   ", VersionColor, nil, nil, nil, nil, 0.2)
				local version 	= UIF:Text1(data.version or "   ", nil, nil, nil, nil, nil, 0.2)
				local type	 	= UIF:Text1(data.releaseType or "   ", nil, nil, nil, nil, nil, 0.3)
				
				self:AddChild(name)
				self:AddChild(revision)
				self:AddChild(version)
				self:AddChild(type)
				
				if count % 2 == 0 then
					name:SetBackground(db.AlternatingRowColor)
					revision:SetBackground(db.AlternatingRowColor)
					version:SetBackground(db.AlternatingRowColor)
					type:SetBackground(db.AlternatingRowColor)
				end
			end
		end
		
		self:DoLayout()
		self.parent:DoLayout()
	end
end